#include <stdbool.h>
#include <stdlib.h>

#include "olaf_config.h"

Olaf_Config* olaf_config_default(){
	Olaf_Config *config = (Olaf_Config*) malloc(sizeof(Olaf_Config));

	config->audioBlockSize = 512;
	config->audioSampleRate = 8000;
	config->audioStepSize= config->audioBlockSize / 2;

	config->bytesPerAudioSample = 4; //32 bits float

	config->printDebugInfo = false;

	//Number of event points per 256 audio samples (32ms)
	config->maxEventPointsPerBlock=3;
	config->maxEventPoints=60;
	config->eventPointThreshold = 20;
	config->minMagnitude = 0.075;//an event point needs to have at least this magnitude
	config->minContrast=3.1;//an event point needs to have at least this contrast with its surroundings (x times the average in surroundings)

	//the filter used in both frequency as time direction
	//to find maxima
	config->filterSize=15; // 11 * 32 = 350 ms
	config->halfFilterSize=config->filterSize/2;

	//min time distance between two event points for fingerprint
	config->minTimeDistance = 2; // 32ms x 2 (+-64ms) for fingerprint construction
	//max time distance between two event points for fingerprint
	config->maxTimeDistance = config->minTimeDistance + 46; // 32ms * (2 + 16) = 576ms for fingerprint construction
	//min freq distance between two event points for fingerprint
	config->minFreqDistance = 3; //bins for fingerprint construction
	//max freq distance between two event points for fingerprint
	config->maxFreqDistance = config->minFreqDistance + (1<<6); //bins for fingerprint construction
	//number of times a event point is reused
	config->maxFingerprintsPerPoint = 5;

	//to check if fingerprint is actually new
	config->recentFingerprintSize=30;

	config->maxFingerprints=300;

	//maximum number of results
	config->maxResults = 10;

	//minimum 5 aligned matches before reporting match
	config->minMatchCount = 5;

	config->maxResultAge = 450;//7.5 seconds

	config->maxDBResults = 100;

	config->includeOffByOneMatches = false;

	return config;
}

void olaf_config_destroy(Olaf_Config * config){
	free(config);
}
