#include <Arduino.h>
#include <driver/i2s.h>
#include <FastLED.h>
#include <olaf.h>
#include "sound_input.h"
#include <driver/i2s.h>
#include "intensity.h"

//Fingerprinter state
struct Olaf_State{
	float audio[512];
	size_t audioBlockIndex;

	PFFFT_Setup *fftSetup;
	float *fft_in;//fft input
	float *fft_out;//fft output

	Olaf_Config *config;
	Olaf_FP_DB* db;
	Olaf_EP_Extractor *ep_extractor;
	Olaf_FP_Extractor *fp_extractor;
	Olaf_FP_Matcher *fp_matcher;

	struct extracted_event_points * eventPoints;
	struct extracted_fingerprints * fingerprints;
};


uint32_t i2s_audio_samples[256];

float referenceTime = 0;

CRGB leds[30];
bool ledOn[30];
int ledStart[30];

struct Olaf_State state;

void setup() {
  Serial.begin(115200);

	int number_of_leds = 30;
	FastLED.addLeds<WS2813, 23, GRB>(leds, number_of_leds).setCorrection( TypicalLEDStrip );
  FastLED.setBrightness(255);

  for(int i = 0 ; i < number_of_leds ; i++){
    ledOn[i]=false;
    ledStart[i]=0;
    leds[0] = CRGB::Black;
  }


	FastLED.show();

	setupSound();

  //Initialize fingerprinter state
  state.fftSetup = pffft_new_setup(512,PFFFT_REAL);
	state.fft_in = (float*) pffft_aligned_malloc(512*4);//fft input, four bytes per float
	state.fft_out= (float*) pffft_aligned_malloc(512*4);//fft output

	//Get the default configuration
	state.config = olaf_config_default();
	state.db = olaf_fp_db_new(NULL,true);
	state.ep_extractor = olaf_ep_extractor_new(state.config);
	state.fp_extractor = olaf_fp_extractor_new(state.config);
	state.fp_matcher = olaf_fp_matcher_new(state.config,state.db);
}

uint32_t mean(){
	uint64_t sum = 0;
	for(int i = 0 ; i< 256 ; i++){
		sum = sum + i2s_audio_samples[i];
	}
	return (uint32_t) (sum/256);
}

float e(){
	float sum = 0;
	for(int i = 0 ; i< 512 ; i++){
		sum =+ abs(state.audio[i]);
	}
	return sum;
}

void setLedStates(float time){

  int currentIntensity = 0;
  int second = (int) time;

  if( second < intensity_length) currentIntensity = intensity[(int) time];

  int millisecondsOn = 335 - currentIntensity * 10;
  int numberOfLedsOnTarget = currentIntensity / 3;

  int currentMillis = millis();

  int numberOfLedsOnCurrent = 0;

  //put out leds that have been on for too long
  for(int i = 0 ; i < 30 ; i++){
    if(ledOn[i]){
      int randTimeDiff =  millisecondsOn + rand() % 80;
      int diff = currentMillis - ledStart[i];
      if(diff > randTimeDiff){
        leds[i]=CRGB::Black;
        ledOn[i]=false;
        ledStart[i]=0;
      }else{
        numberOfLedsOnCurrent++;
      }
    }
  }

  //put on random leds to reach target number of leds
  while(numberOfLedsOnTarget > numberOfLedsOnCurrent  ){
    int randomLed = rand() % 30;
    if(!ledOn[randomLed]){
      int color = rand() % 4;
      if(color==1){
        leds[randomLed]=CRGB::White;
      }else if(color == 2){
        leds[randomLed]=CRGB::Yellow;
      }else{
        leds[randomLed]=CRGB::Blue;
      }
      ledOn[randomLed]=true;
      ledStart[randomLed]=millis();
      numberOfLedsOnCurrent++;
    }
  }

  FastLED.show();
}

void process_audio_block(){


	setLedStates(referenceTime);


	boolean refTimeUpdated = false;

  /*

	//Serial.printf("t %d \n" ,audio_block_index);

	pffft_transform_ordered(state.fft_setup, state.fft_in, state.fft_out, 0, PFFFT_FORWARD);

	//extract event points
	state.eventpoints  =  olaf_ep_extractor_extract(state.ep_extractor,state.fft_out,state.audio_block_index);

	//Serial.printf("Current event point index %d \n",  eventpoints->eventPointIndex);

	//if there are enough event points
	if(state.eventpoints->eventPointIndex > state.config->eventPointThreshold){

		//combine the event points into fingerprints
		fingerprints = olaf_fp_extractor_extract(fp_extractor,eventpoints,audio_block_index);

		//for(int i = 0 ; i < fingerprints->fingerprintIndex ; i++){
	  //  Serial.printf("%d %d %u,\n",audio_block_index ,fingerprints->fingerprints[i].timeIndex1,  olaf_fp_extractor_hash(fingerprints->fingerprints[i]));
		//}

		referenceTime = olaf_fp_matcher_match(fp_matcher,fingerprints,audio_block_index);


	}

	//increase the audio buffer counter
	audio_block_index++;

  */

  //do the transform
	pffft_transform_ordered(state.fftSetup, state.fft_in, state.fft_out, 0, PFFFT_FORWARD);

	//extract event points
	state.eventPoints  =  olaf_ep_extractor_extract(state.ep_extractor,state.fft_out,state.audioBlockIndex);

	//if there are enough event points
	if(state.eventPoints->eventPointIndex > state.config->eventPointThreshold){

		//combine the event points into fingerprints
		state.fingerprints = olaf_fp_extractor_extract(state.fp_extractor,state.eventPoints,state.audioBlockIndex);

/*
		for(size_t i = 0;  i < state.fingerprints->fingerprintIndex ; i++){
			uint64_t timePart = state.fingerprints->fingerprints[i].timeIndex1;
			uint64_t hashPart = olaf_fp_extractor_hash(state.fingerprints->fingerprints[i]);
			uint64_t completePrint = (hashPart<<32) + timePart;
			printf("%lld\n",completePrint);
		}
		*/

		//Serial.printf("%d %zu\n",state.audioBlockIndex ,state.fingerprints->fingerprintIndex );

		referenceTime = olaf_fp_matcher_match(state.fp_matcher,state.fingerprints,state.audioBlockIndex);
		if(referenceTime != 0) refTimeUpdated = true;
	}

	state.audioBlockIndex++;

	if(referenceTime!=0){
		//285*32
		if(!refTimeUpdated) referenceTime  += 0.032;
		//setLedStates(referenceTime);
		Serial.printf("%.2f \n", referenceTime + 0.42 );
	}
}

void loop() {
  size_t bytes_read = 0;

  i2s_read(I2S_NUM_0, i2s_audio_samples, 256*4, &bytes_read, portMAX_DELAY);

  if(bytes_read > 0){
		//Serial.print(mean());
	//	Serial.print(" ");
	//	Serial.println(bytes_read);
	//	Serial.print(" ");
		//int before = micros();

		//For overlap, shift the old samples to the beginning:
		for(int i = 0 ; i <  state.config->audioStepSize; i++){
			state.audio[i] = state.audio[i+state.config->audioStepSize];
		}

		uint32_t meanValue = mean();
		//convert the uint32_t samples to float
		for(int i = 0 ; i <  state.config->audioStepSize; i++){
			int64_t zeroMeanSample = ((int64_t) i2s_audio_samples[i]) - meanValue;
			float audioSample = ((float) zeroMeanSample) / (6000000.0f);

			//copy the new samples to the end
			state.audio[i+state.config->audioStepSize] = audioSample;
		}

		//copy audio samples to fft input and apply hamming window
		for(int i = 0 ; i <  state.config->audioBlockSize ; i++){
			state.fft_in[i] = state.audio[i] * gaussian_window[i];
		}

		//int before = micros();
		process_audio_block();
		//int after = micros();
		//Serial.printf("micros: %d ", after - before);
	  //Serial.println();
	}
}
